

extract_list_element <- function(i, my_list, var) {
  i %>% lapply(function(x) my_list[[x]][[var]]) %>% lapply(null_as) %>% unlist()
}

# Convert a list with single entries to dataframe
list1_to_df <- function(my_list) {
  
  for(f in names(my_list)) {
    if(is.null(my_list[[f]])) 
      my_list[[f]] <- NA
  }
  
  tibble(key = names(my_list), value = unlist(my_list))
}


# Convert a list of lists to dataframe
# requires that every list have same named elements
list_to_df <- function(my_list, as_character= TRUE, on_empty=NA) {
  
  if(is.null(my_list) || is.na(my_list) || length(my_list) ==0)
    return(on_empty)
  
  if(as_character)
    my_list <- lapply(my_list, lapply, as.character)
  
  dplyr::bind_rows(lapply(my_list, as.tibble))
}

# Write yaml to filename with preferred defaults
# Designed so that read_yaml(write_yaml(y)) == y
write_yaml <- function(y, filename) {
  txt <- yaml::as.yaml(y, column.major = FALSE, indent=2)
  txt <- gsub(": ~",":", txt, fixed=TRUE)
  writeLines(txt, filename)
}

export_to_plain_text <- function(austraits, ver, path= "data") {
  
  dir.create(path, FALSE, TRUE)
  
  # Turn information on build into text
  build_info <- sprintf("This is version %s of the AusTraits database.\n
This version was built using commit %s of the raw data, found at https://github.com/traitecoevo/austraits.build.\n
The dataset was created using the following compute session information:\n%s\n.",  
                        ver,
                        stringr::str_replace(austraits[["build_info"]]$git_SHA , "traitecoevo/austraits/tree", "traitecoevo/austraits.build/tree"),
                        paste(capture.output(print(austraits[["build_info"]]$session_info)), collapse = "\n")
  )
  writeLines(build_info, sprintf("%s/build_info.md", path))
  
  # Save definitions
  write_yaml(austraits[["definitions"]],  sprintf("%s/definitions.yml", path))
  
  # Save references
  RefManageR::WriteBib(austraits$sources, sprintf("%s/sources", path))
  
  # Save tables
  for(v in c("traits","sites", "methods", "excluded_data", "taxonomy", "contributors"))
    readr::write_csv(austraits[[v]], sprintf("%s/%s.csv", path, v))
}


remove_study <- function(id, austraits) {
  
  for(f in c("traits", "sites", "methods", "excluded_data") ) {
    austraits[[f]] <- austraits[[f]] %>% filter(dataset_id !=id)
  }
  
  
  i <- which(names(austraits[["sources"]])==id) 
  austraits[["sources"]] <- austraits[["sources"]][-c(i)] 
  
  austraits
}


austraits_load_plain_text <- function(path=here::here("data")) {
  
  austraits <- list()
  austraits$traits <- read_csv(file.path(path, "traits.csv"), guess_max = 1e6)
  austraits$sites <- read_csv(file.path(path, "sites.csv"), guess_max = 1e6)
  austraits$taxonomy <- read_csv(file.path(path, "taxonomy.csv"), guess_max = 1e6)
  austraits$methods <- read_csv(file.path(path, "methods.csv"), guess_max = 1e6)
  austraits$excluded_data <- read_csv(file.path(path, "excluded_data.csv"), guess_max = 1e6)
  austraits$definitions <- yaml::read_yaml(file.path(path, "definitions.yml"))
  austraits$contributors <- read_csv(file.path(path, "contributors.csv"))
  austraits$sources <- ReadBib(file.path(path, "sources.bib"))
  austraits$build_info <- readLines(file.path(path, "build_info.md"))
  austraits
}


bib_print <- function(bib, .opts = list(first.inits = TRUE, max.names = 1000, style="markdown") ) {

  # set format
  oldopts <- RefManageR::BibOptions(.opts)
  on.exit(RefManageR::BibOptions(oldopts))
  bib %>% 
    RefManageR:::format.BibEntry(.sort = F) %>%
    # HACK: remove some of formatting introduced in line above
    # would be nicer if we could apply csl style
    gsub("[] ", "", ., fixed = TRUE) %>% 
    gsub("\\n", "", .) %>% 
    gsub("DOI:", " doi: ", ., fixed = TRUE) %>% 
    gsub("URL:", " url: ", ., fixed = TRUE) %>% 
    ifelse(tolower(bib$bibtype) == "article",  gsub("In:", " ", .), .)
}


## format a table with kable and default styling
my_kable_styling <- function(...) {
  kableExtra::kable(...) %>%
    kableExtra::kable_styling(..., 
                              bootstrap_options = c("striped", "hover", "condensed", "responsive"), 
                              full_width = FALSE, 
                              position = "left"
    ) %>%
    # hack to add margin to plot
    gsub('style="width: auto ', 'style="margin-left:30px; width: auto ', .)
}
